try:
    import unittest2 as unittest
except:
    import unittest
from pyrr import trig


class test_trig(unittest.TestCase):
    def test_import(self):
        import pyrr
        pyrr.trig
        from pyrr import trig

    def test_aspec_ratio(self):
        self.assertEqual(trig.aspect_ratio(1920, 1080), 1920./1080.)

    @unittest.skip('Need a test here')
    def test_calculate_fov(self):
        pass

    @unittest.skip('Need a test here')
    def test_calculate_zoom(self):
        pass

    @unittest.skip('Need a test here')
    def test_calculate_height(self):
        pass

    @unittest.skip('Need a test here')
    def test_calculate_plane_size(self):
        pass

if __name__ == '__main__':
    unittest.main()



try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr import line, ray

class test_line(unittest.TestCase):
    def test_import(self):
        import pyrr
        pyrr.line
        from pyrr import line

    def test_create_zeros(self):
        result = line.create_zeros()
        self.assertTrue(np.allclose(result, [[0,0,0],[0,0,0]]))

    def test_create_from_points(self):
        result = line.create_from_points([-1.,0.,0.],[1.,0.,0.])
        self.assertTrue(np.allclose(result, [[-1,0,0],[1,0,0]]))

    def test_create_from_ray(self):
        r = ray.create([0.,0.,0.], [1., 0.,0.])
        result = line.create_from_ray(r)
        self.assertTrue(np.allclose(result, [[0,0,0],[1,0,0]]))

    def test_start(self):
        l = line.create_from_points([-1.,0.,0.],[1.,0.,0.])
        result = line.start(l)
        self.assertTrue(np.allclose(result, [-1,0,0]))

    def test_end(self):
        l = line.create_from_points([-1.,0.,0.],[1.,0.,0.])
        result = line.end(l)
        self.assertTrue(np.allclose(result, [1,0,0]))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""Provide functions for the creation and manipulation of Lines.

A Line data structure is simply a numpy.array with 2 vectors::

    start = numpy.array( [ -1.0, 0.0, 0.0 ] )
    end = numpy.array( [ 1.0, 0.0, 0.0 ] )
    line = numpy.array( [ start, end ] )

Both Lines and Line Segments are defined using the same data structure.
The only difference is how the data is interpreted.

A line is defined by two points but extends infinitely.

A line segment only exists between two points.
It does not extend forever.

The choice to interprete a line as a line or line segment is up to the
function being called. Check the function signature of documentation
to determine how a line will be interpreted.
"""
from __future__ import absolute_import, division, print_function
import numpy as np
from .utils import all_parameters_as_numpy_arrays, parameters_as_numpy_arrays


class index:
    #: The index of the start vector within the line
    start = 0

    #: The index of the end vector within the line
    end = 1


def create_zeros(dtype=None):
    """Creates a line with the start and end at the origin.

    :rtype: numpy.array
    :return: A line with both start and end points at (0,0,0).
    """
    return np.zeros((2,3), dtype=dtype)

def create_from_points(v1, v2, dtype=None):
    """Creates a line from 2 vectors.

    The 2 vectors represent the start and end point of the line.

    :param numpy.array v1: Start point.
    :param numpy.array v2: End point.
    :rtype: numpy.array
    :return: A line extending from v1 to v2.
    """
    return np.array([v1, v2], dtype=dtype)

@all_parameters_as_numpy_arrays
def create_from_ray(ray):
    """Converts a ray to a line.

    The line will extend from 'ray origin -> ray origin + ray direction'.

    :param numpy.array ray: The ray to convert.
    :rtype: numpy.array
    :return: A line beginning at the ray start and extending for 1 unit
        in the direction of the ray.
    """
    # convert ray relative direction to absolute
    # position
    return np.array([ray[0], ray[0] + ray[1]], dtype=ray.dtype)

@all_parameters_as_numpy_arrays
def start(line):
    """Extracts the start point of the line.

    :param numpy.array line: The line to extract the start from.
    :rtype: numpy.array
    :return: The starting point of the line.
    """
    return line[0].copy()

@all_parameters_as_numpy_arrays
def end(line):
    """Extracts the end point of the line.

    :param numpy.array line: The line to extract the end from.
    :rtype: numpy.array
    :return: The ending point of the line.
    """
    return line[1].copy()




# -*- coding: utf-8 -*-
"""Represents a 4x4 Matrix.

The Matrix44 class provides a number of convenient functions and
conversions.
::

    import numpy as np
    from pyrr import Quaternion, Matrix33, Matrix44, Vector4

    m = Matrix44()
    m = Matrix44([[1.,0.,0.,0.],[0.,1.,0.,0.],[0.,0.,1.,0.],[0.,0.,0.,1.]])

    # copy constructor
    m = Matrix44(Matrix44())

    # explicit creation
    m = Matrix44.identity()
    m = Matrix44.from_matrix44(Matrix44())

    # inferred conversions
    m = Matrix44(Quaternion())
    m = Matrix44(Matrix33())

    # multiply matricies together
    m = Matrix44() * Matrix44()

    # extract a quaternion from a matrix
    q = m.quaternion

    # convert from quaternion back to matrix
    m = q.matrix44
    m = Matrix44(q)

    # rotate a matrix by a quaternion
    m = Matrix44.identity() * Quaternion()

    # rotate a vector 4 by a matrix
    v = Matrix44.from_x_rotation(np.pi) * Vector4([1.,2.,3.,1.])

    # undo a rotation
    m = Matrix44.from_x_rotation(np.pi)
    v = m * Vector4([1.,1.,1.,1.])
    # ~m is the same as m.inverse
    v = ~m * v

    # access specific parts of the matrix
    # first row
    m1 = m.m1
    # first element, first row
    m11 = m.m11
    # fourth element, fourth row
    m44 = m.m44
    # first row, same as m1
    r1 = m.r1
    # first column
    c1 = m.c1
"""
from __future__ import absolute_import
from numbers import Number
import numpy as np
from multipledispatch import dispatch
from .base import BaseObject, BaseMatrix, BaseMatrix44, BaseQuaternion, BaseVector, NpProxy
from .. import matrix44

class Matrix44(BaseMatrix44):
    _module = matrix44
    _shape = (4,4,)

    # m<c> style access
    #: The first row of this Matrix as a numpy.ndarray.
    m1 = NpProxy(0)
    #: The second row of this Matrix as a numpy.ndarray.
    m2 = NpProxy(1)
    #: The third row of this Matrix as a numpy.ndarray.
    m3 = NpProxy(2)
    #: The fourth row of this Matrix as a numpy.ndarray.
    m4 = NpProxy(3)

    # m<r><c> access
    #: The [0,0] value of this Matrix.
    m11 = NpProxy((0,0))
    #: The [0,1] value of this Matrix.
    m12 = NpProxy((0,1))
    #: The [0,2] value of this Matrix.
    m13 = NpProxy((0,2))
    #: The [0,3] value of this Matrix.
    m14 = NpProxy((0,3))
    #: The [1,0] value of this Matrix.
    m21 = NpProxy((1,0))
    #: The [1,1] value of this Matrix.
    m22 = NpProxy((1,1))
    #: The [1,2] value of this Matrix.
    m23 = NpProxy((1,2))
    #: The [1,3] value of this Matrix.
    m24 = NpProxy((1,3))
    #: The [2,0] value of this Matrix.
    m31 = NpProxy((2,0))
    #: The [2,1] value of this Matrix.
    m32 = NpProxy((2,1))
    #: The [2,2] value of this Matrix.
    m33 = NpProxy((2,2))
    #: The [2,3] value of this Matrix.
    m34 = NpProxy((2,3))
    #: The [3,0] value of this Matrix.
    m41 = NpProxy((3,0))
    #: The [3,1] value of this Matrix.
    m42 = NpProxy((3,1))
    #: The [3,2] value of this Matrix.
    m43 = NpProxy((3,2))
    #: The [3,3] value of this Matrix.
    m44 = NpProxy((3,3))

    # rows
    #: The first row of this Matrix as a numpy.ndarray. This is the same as m1.
    r1 = NpProxy(0)
    #: The second row of this Matrix as a numpy.ndarray. This is the same as m2.
    r2 = NpProxy(1)
    #: The third row of this Matrix as a numpy.ndarray. This is the same as m3.
    r3 = NpProxy(2)
    #: The fourth row of this Matrix as a numpy.ndarray. This is the same as m4.
    r4 = NpProxy(3)

    # columns
    #: The first column of this Matrix as a numpy.ndarray.
    c1 = NpProxy((slice(0,4),0))
    #: The second column of this Matrix as a numpy.ndarray.
    c2 = NpProxy((slice(0,4),1))
    #: The third column of this Matrix as a numpy.ndarray.
    c3 = NpProxy((slice(0,4),2))
    #: The fourth column of this Matrix as a numpy.ndarray.
    c4 = NpProxy((slice(0,4),3))

    ########################
    # Creation
    @classmethod
    def from_matrix33(cls, matrix, dtype=None):
        """Creates a Matrix44 from a Matrix33.
        """
        return cls(matrix44.create_from_matrix33(matrix, dtype))

    @classmethod
    def perspective_projection(cls, fovy, aspect, near, far, dtype=None):
        """Creates a Matrix44 for use as a perspective projection matrix.
        """
        return cls(matrix44.create_perspective_projection_matrix(fovy, aspect, near, far, dtype))

    @classmethod
    def perspective_projection_bounds(cls, left, right, top, bottom, near, far, dtype=None):
        """Creates a Matrix44 for use as a perspective projection matrix.
        """
        return cls(matrix44.create_perspective_projection_matrix_from_bounds(left, right, top, bottom, near, far, dtype))

    @classmethod
    def orthogonal_projection(cls, left, right, top, bottom, near, far, dtype=None):
        """Creates a Matrix44 for use as an orthogonal projection matrix.
        """
        return cls(matrix44.create_orthogonal_projection_matrix(left, right, top, bottom, near, far, dtype))

    @classmethod
    def look_at(cls, eye, target, up, dtype=None):
        """Creates a Matrix44 for use as a lookAt matrix.
        """
        return cls(matrix44.create_look_at(eye, target, up, dtype))

    @classmethod
    def from_translation(cls, translation, dtype=None):
        """Creates a Matrix44 from the specified translation.
        """
        return cls(matrix44.create_from_translation(translation, dtype=dtype))

    def __new__(cls, value=None, dtype=None):
        if value is not None:
            obj = value
            if not isinstance(value, np.ndarray):
                obj = np.array(value, dtype=dtype)

            # matrix33
            if obj.shape == (3,3) or isinstance(obj, Matrix33):
                obj = matrix44.create_from_matrix33(obj, dtype=dtype)
            # quaternion
            elif obj.shape == (4,) or isinstance(obj, Quaternion):
                obj = matrix44.create_from_quaternion(obj, dtype=dtype)
        else:
            obj = np.zeros(cls._shape, dtype=dtype)
        obj = obj.view(cls)
        return super(Matrix44, cls).__new__(cls, obj)

    ########################
    # Basic Operators
    @dispatch(BaseObject)
    def __add__(self, other):
        self._unsupported_type('add', other)

    @dispatch(BaseObject)
    def __sub__(self, other):
        self._unsupported_type('subtract', other)

    @dispatch(BaseObject)
    def __mul__(self, other):
        self._unsupported_type('multiply', other)

    @dispatch(BaseObject)
    def __truediv__(self, other):
        self._unsupported_type('divide', other)

    @dispatch(BaseObject)
    def __div__(self, other):
        self._unsupported_type('divide', other)

    def __invert__(self):
        return self.inverse

    ########################
    # Matrices
    @dispatch((BaseMatrix, np.ndarray, list))
    def __add__(self, other):
        return Matrix44(super(Matrix44, self).__add__(Matrix44(other)))

    @dispatch((BaseMatrix, np.ndarray, list))
    def __sub__(self, other):
        return Matrix44(super(Matrix44, self).__sub__(Matrix44(other)))

    @dispatch((BaseMatrix, np.ndarray, list))
    def __mul__(self, other):
        return Matrix44(matrix44.multiply(Matrix44(other), self))

    ########################
    # Quaternions
    @dispatch(BaseQuaternion)
    def __mul__(self, other):
        m = other.matrix44
        return self * m

    ########################
    # Vectors
    @dispatch(BaseVector)
    def __mul__(self, other):
        return type(other)(matrix44.apply_to_vector(self, other))

    ########################
    # Number
    @dispatch((Number, np.number))
    def __add__(self, other):
        return Matrix44(super(Matrix44, self).__add__(other))

    @dispatch((Number, np.number))
    def __sub__(self, other):
        return Matrix44(super(Matrix44, self).__sub__(other))

    @dispatch((Number, np.number))
    def __mul__(self, other):
        return Matrix44(super(Matrix44, self).__mul__(other))

    @dispatch((Number, np.number))
    def __truediv__(self, other):
        return Matrix44(super(Matrix44, self).__truediv__(other))

    @dispatch((Number, np.number))
    def __div__(self, other):
        return Matrix44(super(Matrix44, self).__div__(other))

    ########################
    # Methods and Properties
    @property
    def matrix33(self):
        """Returns a Matrix33 representing this matrix.
        """
        return Matrix33(self)

    @property
    def matrix44(self):
        """Returns the Matrix44.

        This can be handy if you're not sure what type of Matrix class you have
        but require a Matrix44.
        """
        return self

    @property
    def quaternion(self):
        """Returns a Quaternion representing this matrix.
        """
        return Quaternion(self)

from .matrix33 import Matrix33
from .quaternion import Quaternion



try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr import quaternion


class test_quaternion(unittest.TestCase):
    # many of these values are taken from searches on wolfram alpha

    def test_import(self):
        import pyrr
        pyrr.quaternion
        from pyrr import quaternion

    def test_create(self):
        result = quaternion.create()
        np.testing.assert_almost_equal(result, [0., 0., 0., 1.], decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_parameters(self):
        result = quaternion.create(1.0, 2.0, 3.0, 4.0)
        np.testing.assert_almost_equal(result, [1.0, 2.0, 3.0, 4.0], decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_x_rotation(self):
        # 180 degree turn around X axis
        q = quaternion.create_from_x_rotation(np.pi)
        self.assertTrue(np.allclose(q, [1., 0., 0., 0.]))

        # 90 degree rotation around X axis
        q = quaternion.create_from_x_rotation(np.pi / 2.)
        self.assertTrue(np.allclose(q, [np.sqrt(0.5), 0., 0., np.sqrt(0.5)]))

        # -90 degree rotation around X axis
        q = quaternion.create_from_x_rotation(-np.pi / 2.)
        self.assertTrue(np.allclose(q, [-np.sqrt(0.5), 0., 0., np.sqrt(0.5)]))

    def test_create_from_y_rotation(self):
        # 180 degree turn around Y axis
        q = quaternion.create_from_y_rotation(np.pi)
        self.assertTrue(np.allclose(q, [0., 1., 0., 0.]))

        # 90 degree rotation around Y axis
        q = quaternion.create_from_y_rotation(np.pi / 2.)
        self.assertTrue(np.allclose(q, [0., np.sqrt(0.5), 0., np.sqrt(0.5)]))

        # -90 degree rotation around Y axis
        q = quaternion.create_from_y_rotation(-np.pi / 2.)

    def test_create_from_z_rotation(self):
        # 180 degree turn around Z axis
        q = quaternion.create_from_z_rotation(np.pi)
        self.assertTrue(np.allclose(q, [0., 0., 1., 0.]))

        # 90 degree rotation around Z axis
        q = quaternion.create_from_z_rotation(np.pi / 2.)
        self.assertTrue(np.allclose(q, [0., 0., np.sqrt(0.5), np.sqrt(0.5)]))

        # -90 degree rotation around Z axis
        q = quaternion.create_from_z_rotation(-np.pi / 2.)

    def test_create_from_axis_rotation(self):
        # wolfram alpha can be awesome sometimes
        result = quaternion.create_from_axis_rotation([0.57735, 0.57735, 0.57735], np.pi)
        np.testing.assert_almost_equal(result, [5.77350000e-01, 5.77350000e-01, 5.77350000e-01, 6.12323400e-17], decimal=3)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_axis_rotation_non_normalised(self):
        result = quaternion.create_from_axis_rotation([1., 1., 1.], np.pi)
        np.testing.assert_almost_equal(result, [5.77350000e-01, 5.77350000e-01, 5.77350000e-01, 6.12323400e-17], decimal=3)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_matrix_unit(self):
        result = quaternion.create_from_matrix(np.eye(3))
        np.testing.assert_almost_equal(result, [0., 0., 0., 1.], decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_matrix_x(self):
        result = quaternion.create_from_matrix([
            [1., 0., 0.],
            [0., -1., 0.],
            [0., 0., -1.],
        ])
        np.testing.assert_almost_equal(result, [1., 0., 0., 0.], decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_matrix_y(self):
        result = quaternion.create_from_matrix([
            [-1., 0., 0.],
            [0., 1., 0.],
            [0., 0., -1.],
        ])
        np.testing.assert_almost_equal(result, [0., 1., 0., 0.], decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_matrix_z(self):
        result = quaternion.create_from_matrix([
            [-1., 0., 0.],
            [0., -1., 0.],
            [0., 0., 1.],
        ])
        np.testing.assert_almost_equal(result, [0., 0., 1., 0.], decimal=5)
        self.assertTrue(result.dtype == np.float)

    @unittest.skip('Not implemented')
    def test_create_from_eulers(self):
        pass

    @unittest.skip('Not implemented')
    def test_create_from_inverse_of_eulers(self):
        pass

    def test_cross(self):
        q1 = quaternion.create_from_x_rotation(np.pi / 2.0)
        q2 = quaternion.create_from_x_rotation(-np.pi / 2.0)
        result = quaternion.cross(q1, q2)
        np.testing.assert_almost_equal(result, quaternion.create(), decimal=5)

    def test_is_zero_length(self):
        result = quaternion.is_zero_length([1., 0., 0., 0.])
        self.assertFalse(result)

    def test_is_zero_length_zero(self):
        result = quaternion.is_zero_length([0., 0., 0., 0.])
        self.assertTrue(result)

    def test_is_non_zero_length(self):
        result = quaternion.is_non_zero_length([1., 0., 0., 0.])
        self.assertTrue(result)

    def test_is_non_zero_length_zero(self):
        result = quaternion.is_non_zero_length([0., 0., 0., 0.])
        self.assertFalse(result)

    def test_squared_length_identity(self):
        result = quaternion.squared_length([0., 0., 0., 1.])
        np.testing.assert_almost_equal(result, 1., decimal=5)

    def test_squared_length(self):
        result = quaternion.squared_length([1., 1., 1., 1.])
        np.testing.assert_almost_equal(result, 4., decimal=5)

    def test_squared_length_batch(self):
        result = quaternion.squared_length([
            [0., 0., 0., 1.],
            [1., 1., 1., 1.],
        ])
        np.testing.assert_almost_equal(result, [1., 4.], decimal=5)

    def test_length_identity(self):
        result = quaternion.length([0., 0., 0., 1.])
        np.testing.assert_almost_equal(result, 1., decimal=5)

    def test_length(self):
        result = quaternion.length([1., 1., 1., 1.])
        np.testing.assert_almost_equal(result, 2., decimal=5)

    def test_length_batch(self):
        result = quaternion.length([
            [0., 0., 0., 1.],
            [1., 1., 1., 1.],
        ])
        np.testing.assert_almost_equal(result, [1., 2.], decimal=5)

    def test_normalise_identity(self):
        # normalise an identity quaternion
        result = quaternion.normalise([0., 0., 0., 1.])
        np.testing.assert_almost_equal(result, [0., 0., 0., 1.], decimal=5)

    def test_normalise_non_identity(self):
        # normalise an identity quaternion
        result = quaternion.normalise([1., 2., 3., 4.])
        np.testing.assert_almost_equal(result, [1. / np.sqrt(30.), np.sqrt(2. / 15.), np.sqrt(3. / 10.), 2. * np.sqrt(2. / 15.)], decimal=5)

    def test_normalise_batch(self):
        # normalise an identity quaternion
        result = quaternion.normalise([
            [0., 0., 0., 1.],
            [1., 2., 3., 4.],
        ])
        expected = [
            [0., 0., 0., 1.],
            [1. / np.sqrt(30.), np.sqrt(2. / 15.), np.sqrt(3. / 10.), 2. * np.sqrt(2. / 15.)],
        ]
        np.testing.assert_almost_equal(result, expected, decimal=5)

    def test_rotation_angle(self):
        result = quaternion.rotation_angle([5.77350000e-01, 5.77350000e-01, 5.77350000e-01, 6.12323400e-17])
        np.testing.assert_almost_equal(result, np.pi, decimal=5)

    def test_rotation_axis(self):
        result = quaternion.rotation_axis([5.77350000e-01, 5.77350000e-01, 5.77350000e-01, 6.12323400e-17])
        np.testing.assert_almost_equal(result, [0.57735, 0.57735, 0.57735], decimal=5)

    def test_dot_adjacent(self):
        result = quaternion.dot([1., 0., 0., 0.], [0., 1., 0., 0.])
        np.testing.assert_almost_equal(result, 0.0, decimal=5)

    def test_dot_parallel(self):
        result = quaternion.dot([0., 1., 0., 0.], [0., 1., 0., 0.])
        np.testing.assert_almost_equal(result, 1.0, decimal=5)

    def test_dot_angle(self):
        result = quaternion.dot([.2, .2, 0., 0.], [2., -.2, 0., 0.])
        np.testing.assert_almost_equal(result, 0.36, decimal=5)

    def test_dot_batch(self):
        result = quaternion.dot([
            [1., 0., 0., 0.],
            [0., 1., 0., 0.],
            [.2, .2, 0., 0.]
        ], [
            [0., 1., 0., 0.],
            [0., 1., 0., 0.],
            [2., -.2, 0., 0.]
        ])
        expected = [0., 1., 0.36]
        np.testing.assert_almost_equal(result, expected, decimal=5)

    def test_conjugate(self):
        #result = quaternion.conjugate([5.77350000e-01, 5.77350000e-01, 5.77350000e-01, 6.12323400e-17])
        result = quaternion.conjugate([0., 0., 0., 1.])
        np.testing.assert_almost_equal(result, [0., 0., 0., 1.], decimal=5)

    def test_conjugate_rotation(self):
        result = quaternion.conjugate([5.77350000e-01, 5.77350000e-01, 5.77350000e-01, 6.12323400e-17])
        np.testing.assert_almost_equal(result, [-0.57735, -0.57735, -0.57735, 6.12323e-17], decimal=5)

    @unittest.skip('Not implemented')
    def test_power(self):
        pass

    def test_inverse(self):
        result = quaternion.inverse([0., 0., 0., 1.])
        np.testing.assert_almost_equal(result, [0., 0., 0., 1.], decimal=5)

    def test_inverse_rotation(self):
        result = quaternion.inverse([5.77350000e-01, 5.77350000e-01, 5.77350000e-01, 6.12323400e-17])
        np.testing.assert_almost_equal(result, [-0.577351, -0.577351, -0.577351, 6.12324e-17], decimal=5)

    def test_inverse_non_unit(self):
        q = [1, 2, 3, 4]
        result = quaternion.inverse(q)
        expected = quaternion.conjugate(q) / quaternion.length(q)
        np.testing.assert_almost_equal(result, expected, decimal=5)

    def test_negate_unit(self):
        result = quaternion.negate([0., 0., 0., 1.])
        np.testing.assert_almost_equal(result, [0., 0., 0., -1.], decimal=5)

    def test_negate(self):
        result = quaternion.negate([1., 2., 3., 4.])
        np.testing.assert_almost_equal(result, [-1., -2., -3., -4.], decimal=5)

    def test_apply_to_vector_unit_x(self):
        result = quaternion.apply_to_vector([0., 0., 0., 1.], [1., 0., 0.])
        np.testing.assert_almost_equal(result, [1., 0., 0.], decimal=5)

    def test_apply_to_vector_x(self):
        # 180 degree turn around X axis
        q = quaternion.create_from_x_rotation(np.pi)
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [1., 0., 0.]), [1., 0., 0.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 1., 0.]), [0.,-1., 0.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 0., 1.]), [0., 0.,-1.]))

        # 90 degree rotation around X axis
        q = quaternion.create_from_x_rotation(np.pi / 2.)
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [1., 0., 0.]), [1., 0., 0.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 1., 0.]), [0., 0., 1.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 0., 1.]), [0.,-1., 0.]))

        # -90 degree rotation around X axis
        q = quaternion.create_from_x_rotation(-np.pi / 2.)
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [1., 0., 0.]), [1., 0., 0.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 1., 0.]), [0., 0.,-1.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 0., 1.]), [0., 1., 0.]))

    def test_apply_to_vector_y(self):
        # 180 degree turn around Y axis
        q = quaternion.create_from_y_rotation(np.pi)
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [1., 0., 0.]), [-1., 0., 0.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 1., 0.]), [0., 1., 0.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 0., 1.]), [0., 0.,-1.]))

        # 90 degree rotation around Y axis
        q = quaternion.create_from_y_rotation(np.pi / 2.)
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [1., 0., 0.]), [0., 0.,-1.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 1., 0.]), [0., 1., 0.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 0., 1.]), [1., 0., 0.]))

        # -90 degree rotation around Y axis
        q = quaternion.create_from_y_rotation(-np.pi / 2.)
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [1., 0., 0.]), [0., 0., 1.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 1., 0.]), [0., 1., 0.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 0., 1.]), [-1., 0., 0.]))

    def test_apply_to_vector_z(self):
        # 180 degree turn around Z axis
        q = quaternion.create_from_z_rotation(np.pi)
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [1., 0., 0.]), [-1., 0., 0.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 1., 0.]), [0.,-1., 0.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 0., 1.]), [0., 0., 1.]))

        # 90 degree rotation around Z axis
        q = quaternion.create_from_z_rotation(np.pi / 2.)
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [1., 0., 0.]), [0., 1., 0.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 1., 0.]), [-1., 0., 0.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 0., 1.]), [0., 0., 1.]))

        # -90 degree rotation around Z axis
        q = quaternion.create_from_z_rotation(-np.pi / 2.)
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [1., 0., 0.]), [0.,-1., 0.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 1., 0.]), [1., 0., 0.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 0., 1.]), [0., 0., 1.]))

    def test_apply_to_vector_non_unit(self):
        q = quaternion.create_from_x_rotation(np.pi)
        # zero length
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 0., 0.]), [0., 0., 0.]))

        # >1 length
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [2., 0., 0.]), [2., 0., 0.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 2., 0.]), [0.,-2., 0.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [0., 0., 2.]), [0., 0.,-2.]))

    def test_identity(self):
        # https://en.wikipedia.org/wiki/Quaternion
        i = quaternion.create(1., 0., 0., 0.)
        j = quaternion.create(0., 1., 0., 0.)
        k = quaternion.create(0., 0., 1., 0.)
        one = quaternion.create(0., 0., 0., 1.)

        # i * 1 = i
        # j * 1 = j
        # k * 1 = k
        # 1 * i = i
        # 1 * j = j
        # 1 * k = k
        i1 = quaternion.cross(i, one)
        j1 = quaternion.cross(j, one)
        k1 = quaternion.cross(k, one)
        _1i = quaternion.cross(one, i)
        _1j = quaternion.cross(one, j)
        _1k = quaternion.cross(one, k)

        self.assertTrue(np.allclose(i1, _1i, i))
        self.assertTrue(np.allclose(j1, _1j, j))
        self.assertTrue(np.allclose(k1, _1k, k))

        # result = -1
        ii = quaternion.cross(i, i)
        kk = quaternion.cross(k, k)
        jj = quaternion.cross(j, j)
        ijk = quaternion.cross(quaternion.cross(i, j), k)

        self.assertTrue(np.allclose(ii, -one))
        self.assertTrue(np.allclose(jj, -one))
        self.assertTrue(np.allclose(kk, -one))
        self.assertTrue(np.allclose(ijk, -one))

        # ij = k
        # ji = -k
        # jk = i
        # kj = -i
        # ki = j
        # ik = -j

        ij = quaternion.cross(i, j)
        ji = quaternion.cross(j, i)
        jk = quaternion.cross(j, k)
        kj = quaternion.cross(k, j)
        ki = quaternion.cross(k, i)
        ik = quaternion.cross(i, k)

        self.assertTrue(np.allclose(ij, k))
        self.assertTrue(np.allclose(ji, -k))
        self.assertTrue(np.allclose(jk, i))
        self.assertTrue(np.allclose(kj, -i))
        self.assertTrue(np.allclose(ki, j))
        self.assertTrue(np.allclose(ik, -j))

        # -k = ijkk = ij(k^2) = ij(-1)

        ijkk = quaternion.cross(quaternion.cross(ij, k), k)
        ijk2 = quaternion.cross(ij, quaternion.cross(k, k))
        ij_m1 = quaternion.cross(ij, -one)

        self.assertTrue(np.allclose(ijkk, ijk2))
        self.assertTrue(np.allclose(ijk2, ij_m1))


if __name__ == '__main__':
    unittest.main()



try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr import plane

class test_plane(unittest.TestCase):
    def test_import(self):
        import pyrr
        pyrr.plane
        from pyrr import plane

    def test_create(self):
        result = plane.create()
        self.assertTrue(np.allclose(result, [0,0,1,0]))

    def test_create_2(self):
        result = plane.create([1.,0.,0.], 5.)
        self.assertTrue(np.allclose(result, [1.,0.,0.,5.]))

    def test_create_from_points(self):
        vecs = np.array([
            [ 1.0, 0.0, 0.0 ],
            [ 0.0, 1.0, 0.0 ],
            [ 1.0, 1.0, 0.0 ]
        ])
        result = plane.create_from_points(*vecs)
        self.assertTrue(np.allclose(result, [0.,0.,1.,0.]))

    def test_create_from_position(self):
        position = np.array([1.0, 0.0, 0.0])
        normal = np.array([0.0, 3.0, 0.0])
        result = plane.create_from_position(position, normal)
        self.assertTrue(np.allclose(result, [0., 1., 0., 0.]))

    def test_create_xy(self):
        result = plane.create_xy()
        self.assertTrue(np.allclose(result, [0., 0., 1., 0.]))

    def test_create_xy_invert_distance(self):
        result = plane.create_xy(invert=True, distance=2.)
        self.assertTrue(np.allclose(result, [0., 0., -1., 2.]))

    def test_create_xz(self):
        result = plane.create_xz()
        self.assertTrue(np.allclose(result, [0., 1., 0., 0.]))

    def test_create_xz_invert_distance(self):
        result = plane.create_xz(invert=True, distance=2.)
        self.assertTrue(np.allclose(result, [0., -1., 0., 2.]))

    def test_create_yz(self):
        result = plane.create_yz()
        self.assertTrue(np.allclose(result, [1., 0., 0., 0.]))

    def test_create_yz_invert_distance(self):
        result = plane.create_yz(invert=True, distance=2.)
        self.assertTrue(np.allclose(result, [-1., 0., 0., 2.]))

    def test_invert_normal(self):
        p = np.array([1.0, 0.0, 0.0, 1.0])
        result = plane.invert_normal(p)
        self.assertTrue(np.allclose(result, [-1.0, 0.0, 0.0, -1.0]))

    def test_position(self):
        p = plane.create_xz(distance=5.)
        result = plane.position(p)
        self.assertTrue(np.allclose(result, [0.,5.,0.]))

    def test_normal(self):
        p = plane.create_xz(distance=5.)
        result = plane.normal(p)
        self.assertTrue(np.allclose(result, [0.,1.,0.]))

if __name__ == '__main__':
    unittest.main()




try:
    import unittest2 as unittest
except:
    import unittest
from pyrr import integer

class test_integer(unittest.TestCase):
    def test_import(self):
        import pyrr
        pyrr.integer
        from pyrr import integer

    def test_count_bits(self):
        i = 0b010101
        self.assertEqual(integer.count_bits(i), 3)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""Provides common utility functions.
"""
import inspect
from functools import wraps
import numpy as np


def all_parameters_as_numpy_arrays(fn):
    """Converts all of a function's arguments to numpy arrays.

    Used as a decorator to reduce duplicate code.
    """
    # wraps allows us to pass the docstring back
    # or the decorator will hide the function from our doc generator
    @wraps(fn)
    def wrapper(*args, **kwargs):
        args = list(args)
        for i, v in enumerate(args):
            if v is not None:
                args[i] = np.array(v)
        for k,v in kwargs.items():
            if v is not None:
                kwargs[k] = np.array(v)
        return fn(*args, **kwargs)
    return wrapper

def parameters_as_numpy_arrays(*args_to_convert):
    """Converts specific arguments to numpy arrays.

    Used as a decorator to reduce duplicate code.

    Arguments are specified by their argument name.
    For example
    ::
    
        @parameters_as_numpy_arrays('a', 'b', 'optional')
        def myfunc(a, b, *args, **kwargs):
            pass

        myfunc(1, [2,2], optional=[3,3,3])
    """
    def decorator(fn):
        # wraps allows us to pass the docstring back
        # or the decorator will hide the function from our doc generator

        try:
            getfullargspec = inspect.getfullargspec
        except AttributeError:
            getfullargspec = inspect.getargspec

        @wraps(fn)
        def wrapper(*args, **kwargs):
            # get the arguements of the function we're decorating
            fn_args = getfullargspec(fn)

            # convert any values that are specified
            # if the argument isn't in our list, just pass it through

            # convert the *args list
            # we zip the args with the argument names we received from
            # the inspect function
            args = list(args)
            for i, (k, v) in enumerate(zip(fn_args.args, args)):
                if k in args_to_convert and v is not None:
                    args[i] = np.array(v)

            # convert the **kwargs dict
            for k,v in kwargs.items():
                if k in args_to_convert and v is not None:
                    kwargs[k] = np.array(v)

            # pass the converted values to our function
            return fn(*args, **kwargs)
        return wrapper
    return decorator



try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr import euler


class test_euler(unittest.TestCase):
    def test_import(self):
        import pyrr
        pyrr.euler
        from pyrr import euler

    def test_create(self):
        self.assertTrue(np.array_equal(euler.create(), [0., 0., 0.]))
        e = euler.create(roll=1., pitch=2., yaw=3.)
        self.assertEqual(euler.roll(e), 1.)
        self.assertEqual(euler.pitch(e), 2.)
        self.assertEqual(euler.yaw(e), 3.)
        self.assertTrue(np.array_equal(e, [1., 2., 3.]))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""Represents a 3 dimensional Vector.

The Vector3 class provides a number of convenient functions and
conversions.
::

    import numpy as np
    from pyrr import Quaternion, Matrix33, Matrix44, Vector3

    v = Vector3()
    v = Vector3([1.,2.,3.])

    # copy constructor
    v = Vector3(Vector3())

    # add / subtract vectors
    v = Vector3([1.,2.,3.]) + Vector3([4.,5.,6.])

    # rotate a vector by a Matrix
    v = Matrix33.identity() * Vector3([1.,2.,3.])
    v = Matrix44.identity() * Vector3([1.,2.,3.])

    # rotate a vector by a Quaternion
    v = Quaternion() * Vector3([1.,2.,3.])

    # get the dot-product of 2 vectors
    d = Vector3([1.,0.,0.]) | Vector3([0.,1.,0.])

    # get the cross-product of 2 vectors
    x = Vector3([1.,0.,0.]) ^ Vector3([0.,1.,0.])

    # access specific parts of the vector
    # x value
    x,y,z = v.x, v.y, v.z

    # access groups of values as np.ndarray's
    xy = v.xy
    xz = v.xz
    xyz = v.xyz
"""
from __future__ import absolute_import, division
from numbers import Number
import numpy as np
from multipledispatch import dispatch
from .base import BaseObject, BaseVector3, BaseMatrix44, NpProxy
from .. import vector3

# TODO: add < <= > >= == != operators

class Vector3(BaseVector3):
    _module = vector3
    _shape = (3,)

    #: The X value of this Vector.
    x = NpProxy(0)
    #: The Y value of this Vector.
    y = NpProxy(1)
    #: The Z value of this Vector.
    z = NpProxy(2)
    #: The X,Y values of this Vector as a numpy.ndarray.
    xy = NpProxy([0,1])
    #: The X,Y,Z values of this Vector as a numpy.ndarray.
    xyz = NpProxy([0,1,2])
    #: The X,Z values of this Vector as a numpy.ndarray.
    xz = NpProxy([0,2])

    ########################
    # Creation
    @classmethod
    def from_vector4(cls, vector, dtype=None):
        """Create a Vector3 from a Vector4.

        Returns the Vector3 and the W component as a tuple.
        """
        vec, w = vector3.create_from_vector4(vector, dtype)
        return (cls(vec), w)

    def __new__(cls, value=None, w=0.0, dtype=None):
        if value is not None:
            obj = value
            if not isinstance(value, np.ndarray):
                obj = np.array(value, dtype=dtype)

            # matrix44
            if obj.shape in ((4,4,)) or isinstance(obj, BaseMatrix44):
                obj = vector3.create_from_matrix44_translation(obj, dtype=dtype)
        else:
            obj = np.zeros(cls._shape, dtype=dtype)
        obj = obj.view(cls)
        return super(Vector3, cls).__new__(cls, obj)

    ########################
    # Basic Operators
    @dispatch(BaseObject)
    def __add__(self, other):
        self._unsupported_type('add', other)

    @dispatch(BaseObject)
    def __sub__(self, other):
        self._unsupported_type('subtract', other)

    @dispatch(BaseObject)
    def __mul__(self, other):
        self._unsupported_type('multiply', other)

    @dispatch(BaseObject)
    def __truediv__(self, other):
        self._unsupported_type('divide', other)

    @dispatch(BaseObject)
    def __div__(self, other):
        self._unsupported_type('divide', other)

    @dispatch((BaseObject, Number, np.number))
    def __xor__(self, other):
        self._unsupported_type('XOR', other)

    @dispatch((BaseObject, Number, np.number))
    def __or__(self, other):
        self._unsupported_type('OR', other)

    @dispatch((BaseObject, Number, np.number))
    def __ne__(self, other):
        self._unsupported_type('NE', other)

    @dispatch((BaseObject, Number, np.number))
    def __eq__(self, other):
        self._unsupported_type('EQ', other)

    ########################
    # Vectors
    @dispatch((BaseVector3, np.ndarray, list))
    def __add__(self, other):
        return Vector3(super(Vector3, self).__add__(other))

    @dispatch((BaseVector3, np.ndarray, list))
    def __sub__(self, other):
        return Vector3(super(Vector3, self).__sub__(other))

    @dispatch((BaseVector3, np.ndarray, list))
    def __mul__(self, other):
        return Vector3(super(Vector3, self).__mul__(other))

    @dispatch((BaseVector3, np.ndarray, list))
    def __truediv__(self, other):
        return Vector3(super(Vector3, self).__truediv__(other))

    @dispatch((BaseVector3, np.ndarray, list))
    def __div__(self, other):
        return Vector3(super(Vector3, self).__div__(other))

    @dispatch((BaseVector3, np.ndarray, list))
    def __xor__(self, other):
        return self.cross(other)

    @dispatch((BaseVector3, np.ndarray, list))
    def __or__(self, other):
        return self.dot(other)

    @dispatch((BaseVector3, np.ndarray, list))
    def __ne__(self, other):
        return bool(np.any(super(Vector3, self).__ne__(other)))

    @dispatch((BaseVector3, np.ndarray, list))
    def __eq__(self, other):
        return bool(np.all(super(Vector3, self).__eq__(other)))

    ########################
    # Number
    @dispatch((Number,np.number))
    def __add__(self, other):
        return Vector3(super(Vector3, self).__add__(other))

    @dispatch((Number,np.number))
    def __sub__(self, other):
        return Vector3(super(Vector3, self).__sub__(other))

    @dispatch((Number,np.number))
    def __mul__(self, other):
        return Vector3(super(Vector3, self).__mul__(other))

    @dispatch((Number,np.number))
    def __truediv__(self, other):
        return Vector3(super(Vector3, self).__truediv__(other))

    @dispatch((Number,np.number))
    def __div__(self, other):
        return Vector3(super(Vector3, self).__div__(other))

    ########################
    # Methods and Properties
    @property
    def inverse(self):
        """Returns the opposite of this vector.
        """
        return Vector3(-self)

    @property
    def vector3(self):
        return self



# -*- coding: utf-8 -*-
from __future__ import absolute_import
import numpy as np
from pyrr import vector, vector3

class NpProxy(object):
    def __init__(self, index):
        self._index = index

    def __get__(self, obj, cls):
        return obj[self._index]

    def __set__(self, obj, value):
        obj[self._index] = value

class BaseObject(np.ndarray):
    _module = None
    _shape = None
    def __new__(cls, obj):
        # ensure the object matches the required shape
        obj.shape = cls._shape
        return obj

    def _unsupported_type(self, method, other):
        raise ValueError('Cannot {} a {} to a {}'.format(method, type(other).__name__, type(self).__name__))

    ########################
    # Redirect assignment operators
    def __iadd__(self, other):
        self[:] = self.__add__(other)
        return self

    def __isub__(self, other):
        self[:] = self.__sub__(other)
        return self

    def __imul__(self, other):
        self[:] = self.__mul__(other)
        return self

    def __idiv__(self, other):
        self[:] = self.__div__(other)
        return self

class BaseMatrix(BaseObject):
    @classmethod
    def identity(cls, dtype=None):
        """Creates an identity Matrix.
        """
        return cls(cls._module.create_identity(dtype), dtype)

    @classmethod
    def from_eulers(cls, eulers, dtype=None):
        """Creates a Matrix from the specified Euler angles.
        """
        return cls(cls._module.create_from_eulers(eulers, dtype=dtype))

    @classmethod
    def from_quaternion(cls, quat, dtype=None):
        """Creates a Matrix from a Quaternion.
        """
        return cls(cls._module.create_from_quaternion(quat, dtype=dtype))

    @classmethod
    def from_inverse_of_quaternion(cls, quat, dtype=None):
        """Creates a Matrix from the inverse of the specified Quaternion.
        """
        return cls(cls._module.create_from_inverse_of_quaternion(quat, dtype=dtype))

    @classmethod
    def from_scale(cls, scale, dtype=None):
        return cls(cls._module.create_from_scale(scale, dtype=dtype))

    @classmethod
    def from_x_rotation(cls, theta, dtype=None):
        """Creates a Matrix with a rotation around the X-axis.
        """
        return cls(cls._module.create_from_x_rotation(theta, dtype=dtype))

    @classmethod
    def from_y_rotation(cls, theta, dtype=None):
        return cls(cls._module.create_from_y_rotation(theta, dtype=dtype))

    @classmethod
    def from_z_rotation(cls, theta, dtype=None):
        """Creates a Matrix with a rotation around the Z-axis.
        """
        return cls(cls._module.create_from_z_rotation(theta, dtype=dtype))

    @property
    def inverse(self):
        """Returns the inverse of this matrix.
        """
        return type(self)(self._module.inverse(self))

class BaseVector(BaseObject):
    @classmethod
    def from_matrix44_translation(cls, matrix, dtype=None):
        return cls(cls._module.create_from_matrix44_translation(matrix, dtype))

    def normalise(self):
        self[:] = self.normalised

    @property
    def normalised(self):
        return type(self)(self._module.normalise(self))

    @property
    def squared_length(self):
        return self._module.squared_length(self)

    @property
    def length(self):
        return self._module.length(self)

    @length.setter
    def length(self, length):
        self[:] = vector.set_length(self, length)

    def dot(self, other):
        return vector.dot(self, type(self)(other))

    def cross(self, other):
        return type(self)(vector3.cross(self[:3], other[:3]))

    def interpolate(self, other, delta):
        return type(self)(vector.interpolate(self, type(self)(other), delta))

    def normal(self, v2, v3, normalise_result=True):
        return type(self)(vector3.generate_normals(self, type(self)(v2), type(self)(v3), normalise_result))

class BaseQuaternion(BaseObject):
    pass

# pre-declarations to prevent circular imports
class BaseMatrix33(BaseMatrix):
    pass

class BaseMatrix44(BaseMatrix):
    pass

class BaseVector3(BaseVector):
    pass

class BaseVector4(BaseVector):
    pass



try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr import vector, vector3, vector4


class test_vector(unittest.TestCase):
    def test_import(self):
        import pyrr
        pyrr.vector
        from pyrr import vector

    def test_normalise_single_vector(self):
        result = vector3.normalise([1.,1.,1.])
        np.testing.assert_almost_equal(result, [0.57735, 0.57735, 0.57735], decimal=5)

    def test_normalise_batch(self):
        result = vector3.normalise([
            [1.,1.,1.],
            [-1.,-1.,-1.],
            [0.,2.,7.],
        ])
        expected = [
            [0.57735, 0.57735, 0.57735],
            [-0.57735,-0.57735,-0.57735],
            [0., 0.274721, 0.961524],
        ]
        np.testing.assert_almost_equal(result, expected, decimal=5)

    def test_squared_length_single_vector(self):
        result = vector3.squared_length([1.,1.,1.])
        np.testing.assert_almost_equal(result, 3., decimal=5)

    def test_squared_length_batch(self):
        result = vector3.squared_length([
            [1.,1.,1.],
            [-1.,-1.,-1.],
            [0.,2.,7.],
        ])
        expected = [
            3.,
            3.,
            53.,
        ]
        np.testing.assert_almost_equal(result, expected, decimal=5)

    def test_length_vector3(self):
        result = vector3.length([1.,1.,1.])
        np.testing.assert_almost_equal(result, 1.73205, decimal=5)

    def test_length_vector4(self):
        result = vector3.length([1.,1.,1.,1.])
        np.testing.assert_almost_equal(result, 2., decimal=5)

    def test_length_vector3_batch(self):
        result = vector3.length([
            [1.,1.,1.],
            [-1.,-1.,-1.],
            [0.,2.,7.],
        ])
        expected = [
            1.73205,
            1.73205,
            7.28011,
        ]
        np.testing.assert_almost_equal(result, expected, decimal=5)

    def test_set_length_vector3(self):
        result = vector3.set_length([1.,1.,1.],2.)
        expected = [1.15470,1.15470,1.15470]
        np.testing.assert_almost_equal(result, expected, decimal=5)

    def test_set_length_vector4(self):
        result = vector4.set_length([1.,1.,1.,1.],2.)
        expected = [1.,1.,1.,1.]
        np.testing.assert_almost_equal(result, expected, decimal=5)

    def test_set_length_batch_vector(self):
        result = vector3.set_length([
            [1.,1.,1.],
            [-1.,-1.,-1.],
            [0.,2.,7.],
            ], 2.0)
        expected = [
            [1.15470,1.15470,1.15470],
            [-1.15470,-1.15470,-1.15470],
            [0.,0.54944,1.92304],
        ]
        np.testing.assert_almost_equal(result, expected, decimal=5)

    def test_dot_adjacent(self):
        result = vector3.dot([1.,0.,0.], [0.,1.,0.])
        np.testing.assert_almost_equal(result, 0.0, decimal=5)

    def test_dot_parallel(self):
        result = vector3.dot([0.,1.,0.], [0.,1.,0.])
        np.testing.assert_almost_equal(result, 1.0, decimal=5)

    def test_dot_angle(self):
        result = vector3.dot([.2,.2,0.], [2.,-.2,0.])
        np.testing.assert_almost_equal(result, 0.36, decimal=5)

    def test_dot_batch(self):
        result = vector3.dot([
            [1.,0.,0.],
            [0.,1.,0.],
            [.2,.2,0.]
        ],[
            [0.,1.,0.],
            [0.,1.,0.],
            [2.,-.2,0.]
        ])
        expected = [0.,1.,0.36]
        np.testing.assert_almost_equal(result, expected, decimal=5)

    def test_interoplation(self):
        result = vector3.interpolate([0.,0.,0.], [1.,1.,1.], 0.5)
        np.testing.assert_almost_equal(result, [.5,.5,.5], decimal=5)

        result = vector3.interpolate([0.,0.,0.], [2.,2.,2.], 0.5)
        np.testing.assert_almost_equal(result, [1.,1.,1.], decimal=5)

if __name__ == '__main__':
    unittest.main()



try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr import ray


class test_ray(unittest.TestCase):
    def test_import(self):
        import pyrr
        pyrr.ray
        from pyrr import ray

    def test_create(self):
        result = ray.create([0.,0.,0.],[0.,0.,1.])
        np.testing.assert_almost_equal(result, [[0.,0.,0.],[0.,0.,1.]], decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_dtype(self):
        result = ray.create([0,0,0],[0,0,1], dtype=np.int)
        np.testing.assert_almost_equal(result, [[0,0,0],[0,0,1]], decimal=5)
        self.assertTrue(result.dtype == np.int)

    def test_create_from_line(self):
        result = ray.create_from_line([
            [0.,10.,0.],
            [10.,10.,0.]
        ])
        np.testing.assert_almost_equal(result, [[0.,10.,0.],[1.,0.,0.]], decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_invert(self):
        result = ray.invert([[0.,10.,0.],[1.,0.,0.]])
        np.testing.assert_almost_equal(result, [[0.,10.,0.],[-1.,0.,0.]], decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_position(self):
        result = ray.position([[0.,10.,0.],[1.,0.,0.]])
        np.testing.assert_almost_equal(result, [0.,10.,0.], decimal=5)

    def test_direction(self):
        result = ray.direction([[0.,10.,0.],[1.,0.,0.]])
        np.testing.assert_almost_equal(result, [1.,0.,0.], decimal=5)


if __name__ == '__main__':
    unittest.main()

